@@ -14,7 +14,6 @@ setClassUnion("vectorORfactor", c("vector", "factor"))
 setClass("DataTable", representation("VIRTUAL"))
 setClassUnion("DataTableORNULL", c("DataTable", "NULL"))
 
-
 setClass("Vector",
     contains="Annotated",
     representation(
@@ -36,6 +35,7 @@ setMethod("showAsCell", "ANY", function(object) {
   if (length(dim(object)) > 2)
     dim(object) <- c(nrow(object), prod(tail(dim(object), -1)))
   if (NCOL(object) > 1) {
+    class(object) <- setdiff(class(object), "AsIs")
     df <- as.data.frame(object[, head(seq_len(ncol(object)), 3), drop = FALSE])
     attempt <- do.call(paste, df)
     if (ncol(object) > 3)
@@ -143,6 +143,7 @@ setGeneric("rename", function(x, value, ...) standardGeneric("rename"))
 setMethod("rename", "vector", .renameVector)
 setMethod("rename", "Vector", .renameVector)
 
+setMethod("anyNA", "Vector", function(x) any(is.na(x)))
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Validity.
@@ -206,12 +207,12 @@ setValidity2("Vector", .valid.Vector)
 
 setMethod("extractROWS", "NULL", function(x, i) NULL)
 
-.extractROWSFromArray <- function(x, i) {
+.extractROWSWithBracket <- function(x, i) {
   if (is(i, "Ranges"))
-    i <- extractROWS(seq_len(nrow(x)), i)
+    i <- extractROWS(seq_len(NROW(x)), i)
   ## dynamically call [i,,,..,drop=FALSE] with as many "," as length(dim)-1
-  i <- normalizeSingleBracketSubscript(i, x, byrow=TRUE)
   ndim <- max(length(dim(x)), 1L)
+  i <- normalizeSingleBracketSubscript(i, x, byrow = ndim > 1L)
   args <- rep(alist(foo=), ndim)
   names(args) <- NULL
   args[[1]] <- i
@@ -222,7 +223,7 @@ setMethod("extractROWS", "NULL", function(x, i) NULL)
 setMethod("extractROWS", "matrix", function(x, i) {
   if (missing(i))
     return(x)
-  return(.extractROWSFromArray(x, i))
+  return(.extractROWSWithBracket(x, i))
 })
 
 setMethod("extractROWS", "vectorORfactor",
@@ -245,6 +246,14 @@ setMethod("extractROWS", "vectorORfactor",
     }
 )
 
+setMethod("extractROWS", "ANY",
+          function(x, i)
+          {
+            if (missing(i))
+              return(x)
+            .extractROWSWithBracket(x, i)
+          })
+
 setMethod("[", "Vector",
     function(x, i, j, ..., drop=TRUE)
     {
@@ -315,43 +324,11 @@ setReplaceMethod("[", "Vector",
 
 ### Old stuff.
 
-setGeneric("seqselect", signature="x",
-    function(x, start=NULL, end=NULL, width=NULL)
-        standardGeneric("seqselect")
-)
+seqselect <- function(x, start=NULL, end=NULL, width=NULL) .Defunct()
 
-setMethod("seqselect", "ANY",
-    function(x, start=NULL, end=NULL, width=NULL)
-    {
-        .Deprecated(msg="seqselect() is deprecated.")
-        if (!is.null(end) || !is.null(width))
-            start <- IRanges(start=start, end=end, width=width)
-        extractROWS(x, start)
-    }
-)
+`seqselect<-` <- function(x, start=NULL, end=NULL, width=NULL, value) .Defunct()
 
-setGeneric("seqselect<-", signature="x",
-    function(x, start=NULL, end=NULL, width=NULL, value)
-        standardGeneric("seqselect<-")
-)
-
-setReplaceMethod("seqselect", "ANY",
-    function(x, start=NULL, end=NULL, width=NULL, value)
-    {
-        .Deprecated(msg="seqselect() is deprecated.")
-        if (!is.null(end) || !is.null(width))
-            start <- IRanges(start=start, end=end, width=width)
-        replaceROWS(x, start, value)
-    }
-)
-
-subsetByRanges <- function(x, i)
-{
-    .Deprecated("extractROWS")
-    if (!is(i, "Ranges"))
-        stop("'i' must be a Ranges object")
-    extractROWS(x, i)
-}
+subsetByRanges <- function(x, i) .Defunct()
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -422,7 +399,7 @@ window.factor <- window.Vector
 setMethod("window", "factor", window.factor)
 
 ### S3/S4 combo for window.NULL
-window.NULL <- window.Vector
+window.NULL <- function(x, ...) NULL
 setMethod("window", "NULL", window.NULL)
 
 ### S3/S4 combo for window<-.Vector
@@ -481,12 +458,16 @@ setMethod("rep.int", "Vector",
     function(x, times) x[rep.int(seq_len(length(x)), times)]
 )
 
+setGeneric("fixedColumnNames", function(x) standardGeneric("fixedColumnNames"))
+
+setMethod("fixedColumnNames", "ANY", function(x) character())
+
 setMethod("subset", "Vector",
-          function(x, subset, select, drop = FALSE, ...) {
+          function(x, subset, select, drop = FALSE) {
             if (missing(subset)) 
               i <- TRUE
             else {
-              i <- eval(substitute(subset), mcols(x), parent.frame(2))
+              i <- safeEval(substitute(subset), x, top_prenv(subset))
               i <- try(as.logical(i), silent = TRUE)
               if (inherits(i, "try-error")) 
                 stop("'subset' must be coercible to logical")
@@ -495,12 +476,53 @@ setMethod("subset", "Vector",
             if (!missing(select)) {
               nl <- as.list(seq_len(ncol(mcols(x))))
               names(nl) <- colnames(mcols(x))
-              j <- eval(substitute(select), nl, parent.frame(2))
+              j <- eval(substitute(select), nl, top_prenv(select))
               mcols(x) <- mcols(x)[,j,drop=FALSE]
             }
             x[i, drop = drop]
           })
 
+safeEval <- function(expr, envir, enclos) {
+  expr <- eval(call("bquote", expr, enclos))
+  if (!identical(enclos, .GlobalEnv)) {
+    enclos <- makeGlobalWarningEnv(expr, envir, enclos)
+  }
+  eval(expr, envir, enclos)
+}
+
+makeGlobalWarningEnv <- function(expr, envir, enclos) {
+  globals <- setdiff(all.names(expr, functions=FALSE), names(envir))
+  env <- new.env(parent=enclos)
+  lapply(globals, function(g) {
+    makeActiveBinding(g, function() {
+      val <- get(g, enclos)
+      warning("Symbol '", g, "' resolved from calling frame; ",
+              "escape with .(", g, ") for safety.")
+      val
+    }, env)
+  })
+  env
+}
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Evaluating.
+###
+
+setMethod("eval", c("expression", "Vector"),
+          function(expr, envir, enclos = parent.frame())
+          eval(expr, as.env(envir, enclos))
+          )
+
+setMethod("eval", c("language", "Vector"),
+          function(expr, envir, enclos = parent.frame())
+          eval(expr, as.env(envir, enclos))
+          )
+
+setMethod("with", "Vector",
+          function(data, expr, ...)
+          {
+            eval(substitute(expr), data, parent.frame())
+          })
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Coercion.
@@ -532,7 +554,6 @@ setAs("Vector", "vector", function(from) as.vector(from))
 setAs("Vector", "logical", function(from) as.logical(from))
 setAs("Vector", "integer", function(from) as.integer(from))
 setAs("Vector", "numeric", function(from) as.numeric(from))
-setAs("Vector", "double", function(from) as.double(from))
 setAs("Vector", "complex", function(from) as.complex(from))
 setAs("Vector", "character", function(from) as.character(from))
 setAs("Vector", "raw", function(from) as.raw(from))
@@ -547,6 +568,29 @@ as.data.frame.Vector <- function(x, row.names=NULL, optional=FALSE, ...)
 }
 setMethod("as.data.frame", "Vector", as.data.frame.Vector)
 
+setGeneric("as.env", function(x, ...) standardGeneric("as.env"))
+
+makeFixedColumnEnv <- function(x, parent, tform = identity) {
+  env <- new.env(parent=parent)
+  lapply(fixedColumnNames(x), function(nm) {
+    accessor <- get(nm, parent, mode="function")
+    makeActiveBinding(nm, function() {
+      val <- tform(accessor(x))
+      rm(list=nm, envir=env)
+      assign(nm, val, env)
+      val
+    }, env)
+  })
+  env
+}
+
+setMethod("as.env", "Vector", function(x, enclos, tform = identity) {
+  makeFixedColumnEnv(x, as.env(mcols(x), enclos, tform), tform)
+})
+
+setMethod("as.env", "NULL", function(x, enclos, tform = identity) {
+  new.env(parent=enclos)
+})
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Combining.
@@ -607,246 +651,9 @@ setMethod("append", c("Vector", "Vector"),
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Splitting / relisting.
+### mstack()
 ###
 
-setGeneric("splitAsListReturnedClass",
-    function(x) standardGeneric("splitAsListReturnedClass")
-)
-
-setMethod("splitAsListReturnedClass", "ANY",
-    function(x) listClassName("Compressed", class(x))
-)
-
-setMethod("relist", c("ANY", "PartitioningByEnd"),
-    function(flesh, skeleton)
-    {
-        ans_class <- splitAsListReturnedClass(flesh)
-        skeleton_len <- length(skeleton)
-        if (skeleton_len == 0L) {
-            flesh_len2 <- 0L
-        } else {
-            flesh_len2 <- end(skeleton)[skeleton_len]
-        }
-        if (NROW(flesh) != flesh_len2)
-            stop("shape of 'skeleton' is not compatible with 'NROW(flesh)'")
-        if (extends(ans_class, "CompressedList"))
-            return(newCompressedList0(ans_class, flesh, skeleton))
-        if (!extends(ans_class, "SimpleList"))
-            stop("don't know how to split or relist a ", class(flesh),
-                 " object as a ", ans_class, " object")
-        listData <- lapply(skeleton, function(i) extractROWS(flesh, i))
-
-        ## TODO: Once "window" methods have been revisited/tested and
-        ## 'window(flesh, start=start, end=end)' is guaranteed to do the
-        ## right thing for any 'flesh' object (in particular it subsets a
-        ## data.frame-like object along the rows), then replace the line above
-        ## by the code below (which should be more efficient):
-
-        #skeleton_start <- start(skeleton)
-        #skeleton_end <- end(skeleton)
-        #FUN <- function(start, end) window(flesh, start=start, end=end)
-        #names(skeleton_start) <- names(skeleton)
-        #listData <- mapply(FUN, skeleton_start, skeleton_end)
-
-        ## or, if we don't trust mapply():
-
-        #skeleton_start <- start(skeleton)
-        #skeleton_end <- end(skeleton)
-        #X <- seq_len(skeleton_len)
-        #names(X) <- names(skeleton)
-        #listData <- lapply(X, function(i) window(flesh,
-        #                                         start=skeleton_start[i],
-        #                                         end=skeleton_end[i]))
-
-        newList(ans_class, listData)
-    }
-)
-
-setMethod("relist", c("ANY", "List"),
-    function(flesh, skeleton)
-    {
-        relist(flesh, PartitioningByEnd(skeleton))
-    }
-)
-
-setMethod("relist", c("Vector", "list"),
-    function(flesh, skeleton)
-    {
-        relist(flesh, PartitioningByEnd(skeleton))
-    }
-)
-
-.splitAsList_by_listlike <- function(x, f, drop)
-{
-    if (!identical(drop, FALSE))
-        warning("'drop' is ignored when 'f' is a list-like object")
-    if (!is(f, "PartitioningByEnd"))
-        f <- PartitioningByEnd(f)
-    relist(x, f)
-}
-
-.splitAsList_by_integer <- function(x, f, drop)
-{
-    if (length(f) > NROW(x))
-        stop("'f' cannot be longer than 'NROW(x)' when it's an integer vector")
-    idx <- orderInteger(f)
-    tmp <- Rle(f[idx])
-    f <- cumsum(runLength(tmp))
-    names(f) <- as.character(runValue(tmp))
-    if (!identical(drop, FALSE))
-        warning("'drop' is ignored when 'f' is an integer vector")
-    x <- extractROWS(x, idx)
-    f <- PartitioningByEnd(f)
-    relist(x, f)
-}
-
-.splitAsList_by_factor <- function(x, f, drop)
-{
-    x_NROW <- NROW(x)
-    f_len <- length(f)
-    f_levels <- levels(f)
-    f <- as.integer(f)
-    if (f_len > x_NROW)
-        f <- head(f, n=x_NROW)
-    idx <- orderInteger(f)
-    f <- tabulate(f, nbins=length(f_levels))
-    names(f) <- f_levels
-    if (drop)
-        f <- f[f != 0L]
-    f <- cumsum(f)
-    x <- extractROWS(x, idx)
-    f <- PartitioningByEnd(f)
-    relist(x, f)
-}
-
-.splitAsList_by_integer_Rle <- function(x, f, drop)
-{
-    if (length(f) > NROW(x))
-        stop("'f' cannot be longer than data when it's an integer-Rle")
-    f_vals <- runValue(f)
-    f_lens <- runLength(f)
-    idx <- orderInteger(f_vals)
-    xranges <- successiveIRanges(f_lens)[idx]
-    tmp <- Rle(f_vals[idx], f_lens[idx])
-    f <- cumsum(runLength(tmp))
-    names(f) <- as.character(runValue(tmp))
-    if (!identical(drop, FALSE))
-        warning("'drop' is ignored when 'f' is an integer-Rle")
-    x <- extractROWS(x, xranges)
-    f <- PartitioningByEnd(f)
-    relist(x, f)
-}
-
-.splitAsList_by_Rle <- function(x, f, drop)
-{
-    x_NROW <- NROW(x)
-    f_len <- length(f)
-    f_vals <- runValue(f)
-    if (!is.factor(f_vals)) {
-        f_vals <- as.factor(f_vals)
-        if (f_len > x_NROW) {
-            runValue(f) <- f_vals
-            f <- head(f, n=x_NROW)
-            f_vals <- runValue(f)
-        }
-    } else if (f_len > x_NROW) {
-        f <- head(f, n=x_NROW)
-        f_vals <- runValue(f)
-    }
-    f_lens <- runLength(f)
-    f_levels <- levels(f_vals)
-    f_vals <- as.integer(f_vals)
-    idx <- orderInteger(f_vals)
-    xranges <- successiveIRanges(f_lens)[idx]
-    ## Using tabulate2() is 5x faster than doing:
-    ##   f <- integer(length(f_levels))
-    ##   tmp <- Rle(f_vals[idx], f_lens[idx])
-    ##   f[runValue(tmp)] <- runLength(tmp)
-    f <- tabulate2(f_vals, nbins=length(f_levels), weight=f_lens)
-    names(f) <- f_levels
-    if (drop)
-        f <- f[f != 0L]
-    f <- cumsum(f)
-    x <- extractROWS(x, xranges)
-    f <- PartitioningByEnd(f)
-    relist(x, f)
-}
-
-splitAsList <- function(x, f, drop=FALSE)
-{
-    if (!isTRUEorFALSE(drop))
-        stop("'drop' must be TRUE or FALSE")
-    if (is.list(f) || is(f, "List"))
-        return(.splitAsList_by_listlike(x, f, drop))
-    x_NROW <- NROW(x)
-    f_len <- length(f)
-    if (f_len < x_NROW) {
-        if (f_len == 0L)
-            stop("'length(f)' is 0 but 'NROW(x)' is > 0")
-        if (x_NROW %% f_len != 0L)
-            warning("'NROW(x)' is not a multiple of 'length(f)'")
-        f <- rep(f, length.out=x_NROW)
-    }
-    if (is.integer(f))
-        return(.splitAsList_by_integer(x, f, drop))
-    if (is.atomic(f) && is.vector(f))
-        f <- as.factor(f)
-    if (is.factor(f))
-        return(.splitAsList_by_factor(x, f, drop))
-    if (!is(f, "Rle"))
-        stop("'f' must be an atomic vector or a factor (possibly ",
-             "in Rle form), or a list-like object")
-    f_vals <- runValue(f)
-    if (!(is.atomic(f_vals) && is.vector(f_vals)) && !is.factor(f_vals))
-        stop("'f' must be an atomic vector or a factor (possibly ",
-             "in Rle form), or a list-like object")
-    if (is.integer(f_vals))
-        return(.splitAsList_by_integer_Rle(x, f, drop))
-    return(.splitAsList_by_Rle(x, f, drop))
-}
-
-setMethod("split", c("Vector", "ANY"),
-    function(x, f, drop=FALSE, ...) splitAsList(x, f, drop=drop)
-)
-
-setMethod("split", c("ANY", "Vector"),
-          function(x, f, drop=FALSE, ...) splitAsList(x, f, drop=drop)
-)
-
-setMethod("split", c("list", "Vector"),
-          function(x, f, drop=FALSE, ...) split(x, as.vector(f), drop=drop)
-          )
-
-setMethod("split", c("Vector", "Vector"),
-          function(x, f, drop=FALSE, ...) splitAsList(x, f, drop=drop)
-)
-
-`seqsplit<-` <- function(x, f, drop = FALSE, ..., value) {
-  if (!isTRUEorFALSE(drop))
-    stop("'drop' must be TRUE or FALSE")
-  if (length(x) != length(f))
-    stop("Length of 'f' must equal the length of 'x'")
-  ind <- seqsplit(seq_len(length(x)), f, drop = drop)
-  if (length(ind) != length(value))
-    stop("Length of 'value' must equal the length of a split on 'f'")
-  x[unlist(ind, use.names=FALSE)] <- unlist(value, use.names = FALSE)
-  x
-}
-
-setReplaceMethod("split", "Vector", function(x, f, drop = FALSE, ..., value) {
-  seqsplit(x, f, drop = drop, ...) <- value
-  x
-})
-
-multisplit <- function(x, f) {
-  if (!is.list(f) && !is(f, "List"))
-    stop("'f' must be a list")
-  if (length(x) != length(f))
-    stop("Length of 'f' must equal length of 'x'")
-  seqsplit(rep(x, elementLengths(f)), unlist(f, use.names = FALSE))
-}
-
 setGeneric("mstack", function(..., .index.var = "name")
            standardGeneric("mstack"), signature = "...")
 
@@ -854,7 +661,7 @@ setMethod("mstack", "Vector", function(..., .index.var = "name") {
   if (!isSingleString(.index.var))
     stop("'.index.var' must be a single, non-NA string")
   args <- list(...)
-  combined <- do.call(c, unname(args))
+  combined <- compress_listData(args)
   df <- .stack.ind(args, .index.var)
   if (!is.null(mcols(combined)))
     df <- cbind(mcols(combined), df)
@@ -868,12 +675,14 @@ setMethod("mstack", "vector",
             if (!isSingleString(.index.var))
               stop("'.index.var' must be a single, non-NA string")
             args <- list(...)
-            combined <- do.call(c, unname(args))
-            df <- DataFrame(combined, .stack.ind(args, .index.var))
-            colnames(df)[1] <- "value"
+            combined <- compress_listData(args)
+            df <- DataFrame(.stack.ind(args, .index.var), combined)
+            if (ncol(df) == 2L)
+              colnames(df)[2] <- "value"
             df
           })
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Looping methods.
 ###
@@ -1050,3 +859,20 @@ setMethod("aggregate", "data.frame", stats:::aggregate.data.frame)
 
 setMethod("aggregate", "ts", stats:::aggregate.ts)
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Sorting methods.
+###
+
+s4ToVector <- function(x) {
+  if (isS4(x)) {
+    as.vector(x)
+  } else {
+    x
+  }
+}
+
+sortBy <- function(x, formula, decreasing = FALSE) {
+  mf <- model.frame(formula, as.env(x, environment(formula), s4ToVector))
+  x[do.call(order, c(decreasing=decreasing, mf))]
+}
+
