@@ -424,10 +424,11 @@ setMethod("countOverlaps", c("ANY", "Vector"),
     function(query, subject, maxgap = 0L, minoverlap = 1L,
              type = c("any", "start", "end", "within", "equal"), ...)
     {
-        counts <- queryHits(findOverlaps(query, subject, maxgap = maxgap,
-                                         minoverlap = minoverlap, type = type, ...))
-        structure(tabulate(counts, length(query)), names=names(query))
-
+        ov <- findOverlaps(query, subject, maxgap = maxgap,
+                           minoverlap = minoverlap, type = type, ...)
+        ans <- countQueryHits(ov)
+        names(ans) <- names(query)
+        ans
     }
 )
 
@@ -516,151 +517,9 @@ setMethod("countOverlaps", c("RangesList", "RangedData"),
           })
 
 
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### match() is defunct
-###
-
-setMethod("match", c("Views", "Views"),
-    function(x, table, nomatch=NA_integer_, incomparables=NULL)
-    {
-        if (!identical(nomatch, NA_integer_))
-            stop("'nomatch' arg is not supported")
-        msg <- c("match() on Views objects is defunct.\n",
-                 "Please use '",
-                 "findOverlaps(ranges(x), ranges(table), select=\"first\")",
-                 "' instead.")
-        .Defunct(msg=msg)
-    }
-)
-
-setMethod("match", c("Views", "Vector"),
-    function(x, table, nomatch=NA_integer_, incomparables=NULL)
-    {
-        if (!identical(nomatch, NA_integer_))
-            stop("'nomatch' arg is not supported")
-        msg <- c("match() on a Views query is defunct.\n",
-                 "Please use '",
-                 "findOverlaps(ranges(x), table, select=\"first\")",
-                 "' instead.")
-        .Defunct(msg=msg)
-    }
-)
-
-setMethod("match", c("Vector", "Views"),
-    function(x, table, nomatch=NA_integer_, incomparables=NULL)
-    {
-        if (!identical(nomatch, NA_integer_))
-            stop("'nomatch' arg is not supported")
-        msg <- c("match() on a Views subject is defunct.\n",
-                 "Please use '",
-                 "findOverlaps(x, ranges(table), select=\"first\")",
-                 "' instead.")
-        .Defunct(msg=msg)
-    }
-)
-
-setMethod("match", c("RangesList", "RangesList"),
-          function(x, table, nomatch = NA_integer_, incomparables = NULL)
-          {
-            if (!identical(nomatch, NA_integer_))
-                stop("'nomatch' arg is not supported")
-            msg <- c("match() on RangesList objects is defunct.\n",
-                     "Please use '",
-                     "findOverlaps(x, table, select=\"first\", drop=TRUE)",
-                     "' instead.")
-            .Defunct(msg=msg)
-          })
-
-setMethod("match", c("ViewsList", "ViewsList"),
-    function(x, table, nomatch=NA_integer_, incomparables=NULL)
-    {
-        if (!identical(nomatch, NA_integer_))
-            stop("'nomatch' arg is not supported")
-        msg <- c("match() on ViewsList objects is defunct.\n",
-                 "Please use '",
-                 "findOverlaps(ranges(x), ranges(table), select=\"first\", ",
-                 "drop=TRUE)",
-                 "' instead.")
-        .Defunct(msg=msg)
-    }
-)
-
-setMethod("match", c("ViewsList", "Vector"),
-    function(x, table, nomatch=NA_integer_, incomparables=NULL)
-    {
-        if (!identical(nomatch, NA_integer_))
-            stop("'nomatch' arg is not supported")
-        msg <- c("match() on a ViewsList query is defunct.\n",
-                 "Please use '",
-                 "findOverlaps(ranges(x), table, select=\"first\", ",
-                 "drop=TRUE)",
-                 "' instead.")
-        .Defunct(msg=msg)
-    }
-)
-
-setMethod("match", c("Vector", "ViewsList"),
-    function(x, table, nomatch=NA_integer_, incomparables=NULL)
-    {
-        if (!identical(nomatch, NA_integer_))
-            stop("'nomatch' arg is not supported")
-        msg <- c("match() on a ViewsList subject is defunct.\n",
-                 "Please use '",
-                 "findOverlaps(x, ranges(table), select=\"first\", ",
-                 "drop=TRUE)",
-                 "' instead.")
-        .Defunct(msg=msg)
-    }
-)
-
-setMethod("match", c("RangedData", "RangedData"),
-    function(x, table, nomatch = NA_integer_, incomparables = NULL)
-    {
-        if (!identical(nomatch, NA_integer_))
-            stop("'nomatch' arg is not supported")
-        msg <- c("match() on RangedData objects is defunct.\n",
-                 "Please use '",
-                 "findOverlaps(ranges(x), ranges(table), select=\"first\", ",
-                 "drop=TRUE)",
-                 "' instead.")
-        .Defunct(msg=msg)
-    }
-)
-
-setMethod("match", c("RangedData", "RangesList"),
-    function(x, table, nomatch = NA_integer_, incomparables = NULL)
-    {
-        if (!identical(nomatch, NA_integer_))
-            stop("'nomatch' arg is not supported")
-        msg <- c("match() on a RangedData query is defunct.\n",
-                 "Please use '",
-                 "findOverlaps(ranges(x), table, select=\"first\", ",
-                 "drop=TRUE)",
-                 "' instead.")
-        .Defunct(msg=msg)
-    }
-)
-
-setMethod("match", c("RangesList", "RangedData"),
-    function(x, table, nomatch = NA_integer_, incomparables = NULL)
-    {
-        if (!identical(nomatch, NA_integer_))
-            stop("'nomatch' arg is not supported")
-        msg <- c("match() on a RangedData subject is defunct.\n",
-                 "Please use '",
-                 "findOverlaps(x, ranges(table), select=\"first\", ",
-                 "drop=TRUE)",
-                 "' instead.")
-        .Defunct(msg=msg)
-    }
-)
-
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### overlapsAny()
 ###
-### %in% is defunct. Replacing it with %over%.
-###
 
 ### Same args and signature as countOverlaps() and subsetByOverlaps().
 setGeneric("overlapsAny", signature=c("query", "subject"),
@@ -805,28 +664,6 @@ setMethod("overlapsAny", c("RangesList", "RangedData"),
                                                    type="within")
 `%outside%` <- function(query, subject) !overlapsAny(query, subject)
 
-`.%in%.definition` <- function(x, table)
-{
-    msg <- c("%in% between a ", class(x), " and a ", class(table), " object ",
-             "is defunct.\nPlease use 'query %over% subject' instead.")
-    .Defunct(msg=msg)
-}
-
-.signatures <- list(
-    c("Views", "Views"),
-    c("Views", "Vector"),
-    c("Vector", "Views"),
-    c("RangesList", "RangesList"),
-    c("ViewsList", "ViewsList"),
-    c("ViewsList", "Vector"),
-    c("Vector", "ViewsList"),
-    c("RangedData", "RangedData"),
-    c("RangedData", "RangesList"),
-    c("RangesList", "RangedData")
-)
-
-setMethods("%in%", .signatures, `.%in%.definition`)
-
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### subsetByOverlaps()
@@ -906,30 +743,3 @@ setMethod("ranges", "Hits", function(x, query, subject) {
   IRanges(pmax.int(qstart, sstart), pmin.int(send, qend))
 })
 
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### The only reason for defining the methods below is to prevent the default
-### "findMatches" or "countMatches" methods to be called and return something
-### wrong (and the reason they would return something wrong is because they
-### are based on match() which does overlaps instead of equality).
-### TODO: Remove these methods in BioC 2.14 when the "match" methods for all
-### the signatures in '.signatures' are gone.
-
-setMethods("findMatches", .signatures,
-    function(x, table, select=c("all", "first", "last"), ...)
-    {
-        msg <- c("findMatches() between a ", class(x), " and a ",
-                 class(table), " object is not supported")
-        stop(msg)
-    }
-)
-
-setMethods("countMatches", .signatures,
-    function(x, table, ...)
-    {
-        msg <- c("countMatches() between a ", class(x), " and a ",
-                 class(table), " object is not supported")
-        stop(msg)
-    }
-)
-
